import requests

expected_status_codes = [200, 201, 204]


def create_api_call(payload, headers, url, payload_type):
    try:
        if payload_type == "data":
            response = requests.request("POST", url, headers=headers, data=payload)
        else:
            response = requests.request("POST", url, headers=headers, json=payload)
        response.raise_for_status()
        if response.status_code in expected_status_codes:
            return response
        else:
            print(f"Unexpected Status Code: {response.status_code}")
            return None
    except requests.exceptions.RequestException as e:
        print("An error occurred during the API call:", e)
        return None



qa = """
query($query: String!) {
  summaryInsights(engineID: "23343", searchInput: {query: $query, type: [QA]}) {
    data {
      __typename
      ... on QAInsight {
        question
        answers {
          answer
          document {
            id
            title
            abs
            source
            index_name
            __typename
          }
          took
          context
          statistics
          probability
          __typename
        }
        __typename
      }
    }
    __typename
  }
}
"""

documents = """
query($query: String!){
  summaryInsights(engineID: "23343", searchInput: {query: $query, type: [DOCUMENTS]}) {
    data {
      __typename
      ... on QAInsight {
        question
        answers {
          document {
            id
            title
            abs
            source
            index_name
            __typename
          }
          took
          context
          statistics
          probability
          __typename
        }
        __typename
      }
    }
    __typename
  }
}
"""

search_graph = """
query graphInsights($startNode: ID, $startNodeName: String, $nodeTypes: [NodeType], $maxNeighbors: Int) {
  graphInsights(
    engineID: "23343"
    graphInput: {startNode: $startNode, startNodeName: $startNodeName, nodeTypes: $nodeTypes, maxNeighbors: $maxNeighbors}
  ) {
    description
    data {
      __typename
      ... on SubGraph {
        description
        rootNode{
            id
            name
        }
        nodes {
          id
          name
          type
          properties {
            name
            values
            __typename
          }
          __typename
        }
        edges {
          id
          start {
            id
            __typename
          }
          end {
            id
            __typename
          }
          __typename
        }
        __typename
      }
    }
    __typename
  }
}
"""

search_text = """
query($query: String!){
  searchAsYouType(
    searchInput: { searchText: $query, type: SEARCH_LABEL }
  ) {
    description
    type
    data {
      __typename
      ... on SearchLabels {
        searchLabels {
          qid
          text
          nodeTypes
        }
      }
    }
  }
}
"""

execute_score_function = """
query($triples: [[String!]]) { executeScoreFunction(triples: $triples
  )
}
"""

execute_vector_function = """
query($graphIds: [String!]) { executeVectorFunction(graphIds: $graphIds)  
}
"""

execute_nl2sparql = """
query executeNl2Sparql($question: String!) {
  executeNl2Sparql(question: $question) 
}
"""

get_predicates = """
query($label: String!){getPredicates(label: $label)}
"""

get_admet_prediction = """
query ($smiles: [String!], $modelName: String){ admetPredict(smiles: $smiles, modelName: $modelName)
}

"""

advanced_search_query = """
query advancedSearchQuery($query: String) {
  advancedSearchGraph(query: $query) {
    head
    results
    __typename
  }
}
"""


ask_pythia = """
query askPythia($reference: [String!], $response: String!, $question: String) {
  askPythia(reference: $reference, response: $response, question: $question) 
}
"""



from urllib.parse import urlparse


def is_valid_url(text):
    try:
        result = urlparse(text)
        return all([result.scheme, result.netloc])
    except ValueError:
        return False


def create(query, variables):
    payload = {
        'query': query,
        'variables': variables
    }
    return payload



from wisecube_sdk import api_calls, create_payload, create_response, string_query
from wisecube_sdk.model_formats import WisecubeModel, OutputFormat
import json


class WisecubeClient:
    def __init__(self, *args):
        if len(args) == 0:
            open_url = "http://127.0.0.1:5000/api/endpoint"
            self.client = OpenClient(open_url)
        elif len(args) == 1:
            self.client = ApiClient(*args)
        elif len(args) == 3:
            self.client = AuthClient(*args)
        else:
            raise Exception("Invalid args")


class QueryMethods:
    def __init__(self, url, client_id):
        self.url = url
        self.client_id = client_id

    @property
    def output_format(self):
        return getattr(self, '_output_format', OutputFormat.JSON)

    @output_format.setter
    def output_format(self, value):
        if not isinstance(value, OutputFormat):
            raise ValueError("output_format must be a OutputFormat.")
        self._output_format = value

    def get_headers(self):
        raise NotImplementedError("Subclasses must implement get_headers")

    def qa(self, text):
        variables = {
            "query": text
        }
        payload = create_payload.create(string_query.qa, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.qa(response, self.output_format)

    def documents(self, text):
        variables = {
            "query": text
        }
        payload = create_payload.create(string_query.documents, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.documents(response, self.output_format)

    def search_graph(self, text, nr=10):
        if create_payload.is_valid_url(text):
            variables = {
                "maxNeighbours": nr,
                "startNode": text
            }
        else:
            variables = {
                "maxNeighbours": nr,
                "startNodeName": text
            }
        payload = create_payload.create(string_query.search_graph, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.search_graph(response, self.output_format)

    def search_text(self, text):
        variables = {
            "query": text
        }
        payload = create_payload.create(string_query.search_text, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.search_text(response, self.output_format)

    def execute_vector_function(self, graphIds: [str]):
        variables = {
            "graphIds": graphIds
        }
        payload = create_payload.create(string_query.execute_vector_function, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        if response is not None:
            return create_response.execute_vector_function(response, self.output_format)

    def execute_score_function(self, graphIds: [[str]]):
        variables = {
            "triples": graphIds
        }
        payload = create_payload.create(string_query.execute_score_function, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.execute_score_function(response, self.output_format)

    def nl_to_sparql(self, question: str):
        variables = {
            "question": question
        }
        payload = create_payload.create(string_query.execute_nl2sparql, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.nl_2_sparql(response)

    def get_predicates(self, label: str):
        variables = {
            "label": label
        }
        payload = create_payload.create(string_query.get_predicates, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.get_predicates(response, self.output_format)

    def advance_search(self, query: str):
        variables = {
            "query": query
        }
        payload = create_payload.create(string_query.advanced_search_query, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.advanced_search(response, self.output_format)

    def get_admet_predictions(self, smiles: [str], model: WisecubeModel):
        variables = {
            "smiles": smiles,
            "modelName": model.value
        }
        payload = create_payload.create(string_query.get_admet_prediction, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.basic(response)

    def ask_pythia(self, references: [str], response: str, question: str):
        variables = {
            "reference": references,
            "response": response
        }
        if question is not None:
            variables["question"] = question
        payload = create_payload.create(string_query.ask_pythia, variables)
        headers = self.get_headers()
        response = api_calls.create_api_call(payload, headers, self.url, "json")
        return create_response.basic(response)


class OpenClient:
    def __init__(self, url):
        self.url = url


class AuthClient(QueryMethods):
    def __init__(self, username, password, ):
        super().__init__("https://api.wisecube.ai/orpheus/graphql", "1mbgahp6p36ii1jc851olqfhnm")
        self.username = username
        self.password = password

    def create_token(self):
        payload = {
            "AuthParameters": {
                "USERNAME": self.username,
                "PASSWORD": self.password
            },
            "AuthFlow": "USER_PASSWORD_AUTH",
            "ClientId": self.client_id
        }
        headers = {"X-Amz-Target": "AWSCognitoIdentityProviderService.InitiateAuth",
                   "Content-Type": "application/x-amz-json-1.1"
                   }
        cognito_url = "https://cognito-idp.us-east-2.amazonaws.com/"

        response = api_calls.create_api_call(payload, headers, cognito_url, "json")

        token = json.loads(response.text)["AuthenticationResult"]["AccessToken"]

        return token

    def get_headers(self):
        return {
            'Authorization': 'Bearer {}'.format(self.create_token()),
            'Content-Type': 'application/json',
        }


class ApiClient(QueryMethods):
    def __init__(self, api_key):
        super().__init__("https://api.wisecube.ai/orpheus/graphql", "1mbgahp6p36ii1jc851olqfhnm")
        self.api_key = api_key

    def get_headers(self):
        return {
            'Content-Type': 'application/json',
            'x-api-key': self.api_key
        }



from enum import Enum
import pandas as pd


class WisecubeModel(Enum):
    BBB = "BBB"
    LOGS = "logS"
    CYP2CI9I = "CYP2CI9i"
    LOGD7 = "LogD7"
    PGPI = "PGPi"
    PGPS = "PGPs"
    HIA = "HIA"
    F20 = "F20"
    F30 = "F30"
    PPB = "PPB"
    VD = "VD"
    CYPIA2I = "CYPIA2i"
    CYPIA2S = "CYPIA2s"
    CYP3A4I = "CYP3A4i"
    CYP3A4S = "CYP3A4s"
    CYP2C9I = "CYP2C9i"
    CYP2C9S = "CYP2C9s"
    CYP2C19S = "CYP2C19s"
    CYP2D6I = "CYP2D6i"
    CYP2D6S = "CYP2D6s"
    CL = "CL"
    AMES = "Ames"
    DILI = "DILI"
    SKINS = "SkinS"
    CACO2 = "Caco2"
    THALF = "THALF"
    HERG = "hERG"
    HHT = "HHT"


class OutputFormat(Enum):
    JSON = "JSON"
    PANDAS = "PANDAS"



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()


setuptools.setup(
    name="wisecube",
    version="0.0.5",
    author="Wisecube",
    author_email="info@wisecube.ai",
    description="Wisecube SDK for graph Search",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/wisecubeai/wisecube-python-sdk",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=required,
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    extras_require={
        "pandas": ["pandas==1.5.3"],
    }
)


from wisecube_sdk.model_formats import OutputFormat
import pandas as pd


def basic(response):
    response = response.json()
    return response


def qa(response, output_format: OutputFormat):
    response = response.json()
    if output_format == OutputFormat.JSON:
        return response, None
    documents = pd.DataFrame(response["data"]["summaryInsights"][0]["data"]["answers"][0]["document"])
    answer = response["data"]["summaryInsights"][0]["data"]["answers"][0]["answer"]
    return answer, documents


def documents(response, output_format: OutputFormat):
    response = response.json()
    if output_format == OutputFormat.JSON:
        return response
    return pd.DataFrame(response["data"]["summaryInsights"][0]["data"]["answers"][0]["document"])


def search_graph(response, output_format: OutputFormat):
    response = response.json()
    if output_format == OutputFormat.JSON:
        return response
    nodes = pd.DataFrame(response["data"]["graphInsights"]["data"]["nodes"])
    nodes = pd.concat((pd.DataFrame([response["data"]["graphInsights"]["data"]["rootNode"]]), nodes))
    edges = pd.DataFrame(response["data"]["graphInsights"]["data"]["edges"])
    return nodes, edges


def search_text(response, output_format: OutputFormat):
    response = response.json()
    if output_format == OutputFormat.JSON:
        return response
    return pd.DataFrame(response["data"]["searchAsYouType"]["data"]["searchLabels"])


def execute_vector_function(response, output_format: OutputFormat):
    response = response.json()
    if output_format == OutputFormat.JSON:
        return response
    return pd.DataFrame(response["data"]["executeVectorFunction"])


def execute_score_function(response, output_format: OutputFormat):
    response = response.json()
    if output_format == OutputFormat.JSON:
        return response
    return pd.DataFrame(response["data"]["executeScoreFunction"], columns=["subject", "predicate", "object", "score"])


def get_predicates(response, output_format: OutputFormat):
    response = response.json()
    if output_format == OutputFormat.JSON:
        return response
    return pd.DataFrame(response["data"]["getPredicates"])


def nl_2_sparql(response):
    response = response.json()
    try:
        return response["data"]["executeNl2Sparql"]["SPARQL"]
    except Exception as e:
        print(e)
    return response


def advanced_search(response, output_format: OutputFormat):
    response = response.json()
    if output_format == OutputFormat.JSON:
        return response
    columns = response["data"]["advancedSearchGraph"]["head"]["vars"]
    rows = [{k: d["value"] for k, d in r.items()} for r in
            response["data"]["advancedSearchGraph"]["results"]["bindings"]]
    return pd.DataFrame(rows)[columns]
